import re
import generic_design_patterns as gdp

from . import base


class AbstractAnnotator(gdp.chain.ChainNodePlugin):
    def handle(self, rule: base.RuleSetting, row: base.AnnotatedRow):
        row.highlights.append(rule.highlight)
        row.horizontal_style = rule.highlight.major_style


class HorizontalAnnotator(AbstractAnnotator):
    def check(self, rule: base.RuleSetting, row: base.AnnotatedRow):
        re_conditions = [
            rule.recognizer.type == base.RecognizerType.re,
            bool(re.search(str(rule.recognizer.value), row.text))
        ]

        row_conditions = [
            rule.recognizer.type == base.RecognizerType.row,
            rule.highlight.orientation == base.HighlightOrientation.horizontal,
            self._check_row_number(row, rule)
        ]

        conditions = [all(re_conditions), all(row_conditions)]

        return any(conditions)

    def _check_row_number(self, row, rule):
        try:
            return row.number in rule.recognizer.value
        except Exception as e:
            return False


class VerticalFirstRowAnnotator(AbstractAnnotator):
    def check(self, rule: base.RuleSetting, row: base.AnnotatedRow):
        conditions = [
            rule.recognizer.type == base.RecognizerType.row,
            rule.highlight.orientation != base.HighlightOrientation.horizontal,
            self._check_row_number(row, rule)
        ]

        return all(conditions)

    def handle(self, rule: base.RuleSetting, row: base.AnnotatedRow):
        row.highlights.append(rule.highlight)
        row.vertical_style[rule.highlight.orientation] = rule.highlight.major_style
        self._set_rowspan(rule, row)

    def _set_rowspan(self, rule, row):
        current_rs = row.rowspan[rule.highlight.orientation]
        rs = max([current_rs, rule.recognizer.value[-1] - rule.recognizer.value[0]])
        row.rowspan[rule.highlight.orientation] = rs


    def _check_row_number(self, row, rule):
        try:
            return row.number == rule.recognizer.value[0]
        except Exception as e:
            return False


class VerticalNextRowAnnotator(AbstractAnnotator):
    def check(self, rule: base.RuleSetting, row: base.AnnotatedRow):
        conditions = [
            rule.recognizer.type == base.RecognizerType.row,
            rule.highlight.orientation != base.HighlightOrientation.horizontal,
            self._check_row_number(row, rule)
        ]

        return all(conditions)

    def handle(self, rule: base.RuleSetting, row: base.AnnotatedRow):
        # row.vertical_style[rule.highlight.orientation] = rule.highlight.major_style
        self._set_rowspan(rule, row)

    def _set_rowspan(self, rule, row):
        current_rs = row.rowspan[rule.highlight.orientation]
        if current_rs > 0:
            rs = current_rs
        else:
            rs = base.continuing_vertical_highlight
        row.rowspan[rule.highlight.orientation] = rs

    def _check_row_number(self, row, rule):
        try:
            return row.number in rule.recognizer.value[1:]
        except Exception as e:
            return False


class RowAnnotator:
    def __init__(self):
        annotator_collectors = [gdp.plugin.SubclassPluginCollector(AbstractAnnotator)]
        self.annotators_chain = gdp.chain.build(annotator_collectors)

    def annotate(self, rules: [base.RuleSetting], rows: [base.AnnotatedRow]):
        for row in rows:
            for rule in rules:
                self.annotators_chain.handle(rule, row)






import enum
import re

from . import base


class RulesParser:
    separator = r"\s{4}\s*"

    def parse(self, raw_rules: [str]):
        try:
            return self._parse_rules(raw_rules)
        except Exception as e:
            raise base.LocomException(f"{e} Parsing rules failed.")

    def _parse_rules(self, raw_rules: [str]):
        rules = []

        for row_number, raw_rule in enumerate(raw_rules, 1):
            try:
                striped_raw_rule = raw_rule.strip()
                if striped_raw_rule == "" or striped_raw_rule[0] == "#":
                    continue
                else:
                    rule = self._parse_rule(raw_rule)
            except Exception as e:
                raise base.LocomException(f"{e} Parsing rule on row {row_number} failed.")
            rules.append(rule)

        return rules

    def _parse_rule(self, raw_rule: str) -> base.RuleSetting:
        parts = re.split(self.separator, raw_rule.strip())
        rule = base.RuleSetting()

        parts = [part.strip() for part in parts]
        parts.extend([""] * (5 - len(parts)))

        rule.recognizer.type = parts[base.RawRuleIndex.recognizer_type]
        rule.recognizer.value = parts[base.RawRuleIndex.recognizer_value]
        rule.highlight.text = parts[base.RawRuleIndex.text]
        rule.highlight.detail = parts[base.RawRuleIndex.detail]

        self._parse_styles(parts[base.RawRuleIndex.styles], rule)
        self._parse_orientation(rule)
        self._validate(rule)
        self._parse_rows(parts[base.RawRuleIndex.recognizer_value], rule)
        return rule

    def _parse_styles(self, combined_style, rule):
        parts = combined_style.split(":")

        parts.extend(["normal"] * (2 - len(parts)))

        rule.highlight.major_style = parts[base.ColorIndex.major_style]
        rule.highlight.minor_style = parts[base.ColorIndex.minor_style]

    def _parse_rows(self, raw_rows, rule):
        # TODO: protect by TRY
        if rule.recognizer.type != base.RecognizerType.re:
            r = []

            for part in raw_rows.split(","):
                sub_parts = part.split("-")

                if len(sub_parts) == 1:
                    r += [int(sub_parts[0])]

                if len(sub_parts) == 2:
                    # TODO: [0] > [1]
                    r += list(range(int(sub_parts[0]), int(sub_parts[1]) + 2))

             # TODO: Side effect! reconizer_value was originaly set in _parse_rule(). Confusing
            rule.recognizer.value = r
            rule.highlight.rows = r

    def _parse_orientation(self, rule):
        if base.HighlightOrientation.left in rule.highlight.major_style:
            self._set_orientation(rule, base.HighlightOrientation.left)

        elif base.HighlightOrientation.right in rule.highlight.major_style:
            self._set_orientation(rule, base.HighlightOrientation.right)

        else:
            self._set_orientation(rule, base.HighlightOrientation.horizontal)

    def _set_orientation(self, rule, orientation):
        rule.highlight.orientation = orientation
        rule.recognizer.orientation = orientation

    def _validate(self, rule):
        if rule.recognizer.type == "re" and rule.highlight.orientation != base.HighlightOrientation.horizontal:
            raise base.LocomException(f"Invalid combination of row recognizer ({rule.recognizer.type}) "
                                      f"and vertical highlight style ({rule.highlight.orientation}).")





import enum


class LocomException(Exception):
    pass


class RawRuleIndex(enum.IntEnum):
    recognizer_type = 0
    recognizer_value = enum.auto()
    styles = enum.auto()
    text = enum.auto()
    detail = enum.auto()


class ColorIndex(enum.IntEnum):
    major_style = 0
    minor_style = enum.auto()


class RuleSetting:
    def __init__(self):
        self.recognizer = RecognizerSetting()
        self.highlight = HighlightSetting()


class RecognizerType:
    re = "re"
    row = "row"


class RecognizerSetting:
    def __init__(self):
        self.type = None
        self.value = None
        # self.orientation = None


class HighlightOrientation:
    horizontal = "horizontal"
    left = "left"
    right = "right"


class HighlightSetting:
    def __init__(self):
        self.orientation = None
        self.major_style = None
        self.minor_style = None
        self.text = None
        self.detail = None


class RenderSetting:
    def __init__(self):
        self.whitespace_protection = None
        self.escape_sequence_protection = None


class AnnotatedRow:
    def __init__(self, number, text):
        self.number = number
        self.text = text
        self.highlights = []
        self.horizontal_style = "normal"
        self.vertical_style = {VerticalColumn.left: "normal", VerticalColumn.right: "normal"}
        self.rowspan = {VerticalColumn.left: 0, VerticalColumn.right: 0}


class VerticalColumn:
    left = "left"
    right = "right"


continuing_vertical_highlight = -1


class RenderElementType:
    title = "TITLE"
    source_file = "SOURCE_FILE"
    description = "DESCRIPTION"
    multi_row_column = "MULTI_ROW_COLUMN"
    row_number_column = "ROW_NUMBER_COLUMN"
    log_column = "LOG_COLUMN"
    comment_column = "COMMENT_COLUMN"
    rows = "ROWS"


class RenderRows(list):
    type = RenderElementType.rows


class RenderElement:
    def __init__(self, type, text):
        self.type = type
        self.text = text


import argparse

from . import cli

# TODO: Auto/gen cli
# TODO: MultiRow & MultiColumn
# TODO: MultiRow RE
# TODO: Code Refactor
# TODO: README # EXAMPLES
# TODO: Protection whitespaces & CLI options
# TODO: Fix: Overlap Bug: (row,10 - 20, green) and (row, 10 - 30, left-green)
# TODO: Fix: Comments layout bug
# TODO: [OPTIONAL] Cli options & Size font (Size levels ...)


def main():
    parser = argparse.ArgumentParser(description="Locom is tool for generation commented log. "
                                                 "The output format is HTML. "
                                                 "For more information visit https://pypi.org/project/locom/.",
                                     formatter_class=argparse.RawTextHelpFormatter)
    subparsers = parser.add_subparsers()

    cli_parser = subparsers.add_parser('cli')
    cli_parser.set_defaults(func=cli.run)

    cli_parser.add_argument("-r", "--rules-file",
                            required=True,
                            help="Rules describes recognition and rendering of rows.")
    cli_parser.add_argument("-i", "--input-file",
                            required=True,
                            help="Input log")
    cli_parser.add_argument("-o", "--output-file",
                            default="",
                            help="If output file is empty filename will be same as input file. "
                                 "Only suffix will be .html.")
    cli_parser.add_argument("-t", "--template",
                            default="dark",
                            help="The template for output html.")
    cli_parser.add_argument("--title",
                            default="",
                            help="The title for html output page.")
    cli_parser.add_argument("--description",
                            default="",
                            help="The description for html output page.")
    cli_parser.add_argument("--row-number-column",
                            default="3",
                            help="The percentage size of row column in output html.")
    cli_parser.add_argument("--log-column",
                            default="64",
                            help="The percentage size of log column in output html.")
    cli_parser.add_argument("--mr-column",
                            default="2",
                            help="The percentage size of multi-row comment columns in output html.")
    cli_parser.add_argument("--cancel-whitespace-protection",
                            default="",
                            help="")
    cli_parser.add_argument("--cancel-escape-sequence-protection",
                            default="",
                            help="")

    generator_parser = subparsers.add_parser('gen')
    generator_parser.set_defaults(func=cli.generator)

    generator_parser.add_argument('--suffixes', type=str, nargs='+', default=["*.txt", "*.log"])

    auto_parser = subparsers.add_parser('auto')
    auto_parser.set_defaults(func=cli.auto)

    auto_parser.add_argument('--suffixes', type=str, nargs='+', default=["*.txt", "*.log"])

    arguments = parser.parse_args()
    arguments.func(arguments)


if __name__ == "__main__":
    main()



import io
import os
import base64
from PIL import Image
import generic_design_patterns as gdp


def file_suffix(path):
    return path.split(".")[-1].lower()


class AbstractDetailRender(gdp.chain.ChainNodePlugin):
    suffixes = []

    def description(self):
        return self.__class__.__name__

    def check(self, detail):
        return os.path.exists(detail) and (file_suffix(detail) in self.suffixes)


class TextDetailRender(AbstractDetailRender):
    def check(self, detail):
        # TODO: Exists bad it is not supported format
        return not os.path.exists(detail)

    def handle(self, detail):
        return f"<div>{detail}</div>"


class ImageDetailRender(AbstractDetailRender):
    suffixes = ["jpeg", "jpg", "png", "bmp"]

    def handle(self, detail):
        img = Image.open(detail)

        img_format = file_suffix(detail)
        img_format = "jpeg" if img_format == "jpg" else img_format

        buffer = io.BytesIO()
        img.save(buffer, format=img_format)
        buffer.seek(0)
        img_base64 = base64.b64encode(buffer.read()).decode('ascii')

        return f'<img src="data:image/{img_format};base64,{img_base64}">'


class HtmlDetailRender(AbstractDetailRender):
    suffixes = ["html"]

    def handle(self, detail):
        with open(detail) as detail_file:
            return detail_file.read()


class DetailRender:
    def __init__(self):
        render_collectors = [gdp.plugin.SubclassPluginCollector(AbstractDetailRender)]
        self.renders_chain = gdp.chain.build(render_collectors)

    def render(self, detail):
        return self.renders_chain.handle(detail)


import os

from . import base


class Loader:
    @staticmethod
    def template_directory():
        package_directory = os.path.dirname(os.path.abspath(__file__))
        return os.path.join(package_directory, "template")

    def load(self, template_name: str) -> str:
        template_file = self.template_path(template_name)
        try:
            with open(template_file) as t:
                return t.read()
        except Exception as e:
            raise base.LocomException("Reading template file '%s' failed." % template_file)

    def template_path(self, template_name: str):
        filename = "%s.html" % template_name
        return os.path.join(self.template_directory(), filename)




import generic_design_patterns as gdp


class AbstractElementRender(gdp.chain.ChainNodePlugin):
    def description(self):
        return self.__class__.__name__

    def _target_place(self, element):
        return f"==={element.type}==="



import datetime
import random

from . import abstract
from . import detail_render

from .. import base


class RowsRender(abstract.AbstractElementRender):
    row_template = """
    <tr class="{}">
        {}
        <td><div class="row-number">{}</div></td>
        <td><div class="log">{}</div></td>
        <td>{}</td>
        {}
    </tr>
    """

    def check(self, setting, template, element) -> bool:
        return True if element.type == base.RenderElementType.rows else False

    def handle(self, setting,  template, element) -> str:
        self.rows = element
        self.setting = setting
        output = ""

        for row in self.rows:
            render_content = [sub_row_render.render(setting, row) for sub_row_render in self._sub_renders()]
            output += self.row_template.format(*render_content)

        return template.replace(self._target_place(element), output)

    def _sub_renders(self):
        return [RowStyleRender(),
                LeftVerticalHighlightRender(),
                RowNumberRender(),
                RowLogRender(),
                HorizontalHighlightRender(),
                RightVerticalHighlightRender()]


def fix_escape_sequence(f):
    def wrapper(*args, **kwargs) -> str:
        s = f(*args, **kwargs)
        fixed = s.replace("&", "&amp;")
        fixed = fixed.replace(" ", "&nbsp;")
        fixed = fixed.replace("<", "&lt;")
        fixed = fixed.replace(">", "&gt;")
        return fixed
    return wrapper


class RowStyleRender:
    def render(self, setting, row: base.AnnotatedRow):
        return f"{row.horizontal_style}-row"
        # if len(major_styles):
        #     return "%s-row" % major_styles[-1]
        # else:
        #     return "normal-row"


class RowNumberRender:
    def render(self, setting, row):
        return row.number


class RowLogRender:
    @fix_escape_sequence
    def render(self, setting, row):
        return row.text


class HorizontalHighlightRender:
    expander_template = """<div class="box-comment-expander"></div>"""

    highlight_template = """
            <div class="unbox-comment"><div class="%s-box-comment"><a href="#modal%s">%s</a></div>
                <div id="modal%s" class="modal-box %s-modal-box">
                    <div>
                        <div class="modal-head %s-modal-head">
                            <span class="modal-title %s-modal-title">%s</span>
                            <span class="modal-close-button %s-modal-close-button"><a class="closeWindow" href="#close">X</a></span>
                        </div>
                        <div class="modal-row-view">
                            %s %s
                        </div>
                        <div class="modal-content">%s</div>
                    </div>
                </div>

            </div>
            """

    def render(self, setting, row: base.AnnotatedRow):
        output = ""

        horizontal_highlights = [highlight for highlight in row.highlights if highlight.orientation == base.HighlightOrientation.horizontal]

        for highlight in horizontal_highlights:
                output += self._render_horizontal_highlight(row, highlight)
        output += self.expander_template
        return output

    def _render_horizontal_highlight(self, row, highlight):
        output = ""
        dt = datetime.datetime.now()
        ts = "%s_%s" % (random.randrange(0, 101, 1), int(dt.timestamp() * 1000))

        detail_style = self._detail_style(row, highlight)

        output += self.highlight_template % (
            highlight.minor_style,
            ts,
            highlight.text,
            ts,
            detail_style,
            detail_style,
            detail_style,
            highlight.text,
            detail_style,
            row.number,
            row.text,
            self._detail(highlight)
        )
        return output

    def _detail_style(self, row, highlight):

        if highlight.minor_style == "normal":
            return row.horizontal_style
        else:
            return highlight.minor_style

    def _detail(self, highlight):
        if highlight.detail != "":
            r = detail_render.DetailRender()
            return r.render(highlight.detail)
        else:
            return highlight.detail


class VerticalHighlightRender:
    side = ""
    highlight_template = """
    <div class="%s-box-comment multi-row"><a href="#modal%s">%s</a></div>
        <div id="modal%s" class="modal-box %s-modal-box">
                    <div>
                        <div class="modal-head %s-modal-head">
                            <span class="modal-title %s-modal-title">%s</span>
                            <span class="modal-close-button %s-modal-close-button"><a class="closeWindow" href="#close">X</a></span>
                        </div>
                        <div class="modal-row-view">
                            %s %s
                        </div>
                        <div class="modal-content"></div>
                    </div>
            </div>
    """
    multi_row_template = """
        <td rowspan="%s" class="%s-box-comment">
            %s    
        </td>
    """

    def render(self, setting, row: base.AnnotatedRow):
        highlights = ""

        if row.rowspan[self.side] == base.continuing_vertical_highlight:
            return ""
        if row.rowspan[self.side] == 0:
            return "<td></td>"


        vertical_highlights = [highlight for highlight in row.highlights if highlight.orientation == self.side]

        for highlight in vertical_highlights:
            highlights += self._render_vertical_highlight(row, highlight)

        # if highlights == "":
        #     if row.rowspan[self.side] == base.continuing_vertical_highlight:
        #         return ""
        #     elif row.rowspan[self.side] == 0:
        #         return "<td></td>"
        # else:
        #     return self.multi_row_template % (
        #         row.rowspan[self.side],
        #         row.vertical_style,
        #         highlights
        #     )

        return self.multi_row_template % (
            row.rowspan[self.side],
            fix_style(row.vertical_style[self.side]),
            highlights
        )

    def _render_vertical_highlight(self, row, highlight):
        output = ""
        dt = datetime.datetime.now()
        ts = "%s_%s" % (random.randrange(0, 101, 1), int(dt.timestamp() * 1000))

        detail_style = fix_style(row.vertical_style[self.side])

        output += self.highlight_template % (
            highlight.minor_style,
            ts,
            highlight.text,
            ts,
            detail_style,
            detail_style,
            detail_style,
            highlight.text,
            detail_style,
            "MULTI ROW: ",
            "%s - %s" % (row.number, row.number + row.rowspan[self.side])
        )
        return output


class LeftVerticalHighlightRender(VerticalHighlightRender):
    side = base.HighlightOrientation.left


class RightVerticalHighlightRender(VerticalHighlightRender):
    side = base.HighlightOrientation.right


def fix_style(style):
    return style.replace("left-", "").replace("right-", "")


import os
import glob

from . import base
from . import parser
from . import template
from . import annotator
from . import render


def run(arguments):
    app = App(arguments)
    app.run()


# def relevant_files(suffixes, path="."):
#     base_directory = os.path.abspath(path)
#
#     files = []
#
#     for suffix in suffixes:
#         files += glob.glob(os.path.join(base_directory, suffix))
#
#     return files
#
#
# def rule_file(path):
#     basename = os.path.basename(path)
#     return os.path.join(locom_directory(path), basename)
#
#
# def locom_directory(path="."):
#     dirname = os.path.dirname(path)
#     return os.path.join(dirname, "locom")
#
#
def auto(arguments):
    pass
    # input_files = [InputFile(input_file_path) for input_file_path in CurrentDirectory.input_files(arguments.suffixes)]
#
#
# class CurrentDirectory:
#     @staticmethod
#     def input_files(suffixes):
#         return []
#
#
# class LocomRuleDirectory:
#     @property
#     def path(self):
#         pass
#
#     @staticmethod
#     def create():
#         pass


class InputFile:
    def __init__(self, path):
        self.path = path

    @property
    def rule_file(self):
        return ""


def create_empty_file(path):
    with open(path, 'a'):
        os.utime(path, None)


def generator(arguments):
    pass
    # LocomRuleDirectory.create()
    # input_files = [InputFile(input_file_path) for input_file_path in CurrentDirectory.input_files(arguments.suffixes)]
    #
    # for input_file in input_files:
    #     create_empty_file(input_file.path)


class App:
    def __init__(self, settings):
        self.settings = settings

        self.raw_rules = None
        self.rules = None
        self.rows = None
        self.template = None
        self.output = None

    def run(self):
        self._read_rules_file()
        self._parser_rules()
        self._read_input_file()
        self._read_template_file()
        self._annotate_rows()
        self._render()
        self._write_to_output_file()

    def _read_rules_file(self):
        try:
            with open(self.settings.rules_file) as rf:
                self.raw_rules = rf.readlines()
        except Exception as e:
            raise base.LocomException(f"Reading rules file '{self.settings.rules_file}' failed. {e}")

    def _parser_rules(self):
        rule_parser = parser.RulesParser()
        self.rules = rule_parser.parse(self.raw_rules)

    def _read_input_file(self):
        try:
            with open(self.settings.input_file) as i:
                self.rows = [base.AnnotatedRow(number, text) for number, text in enumerate(i.readlines(), 1)]
        except Exception as e:
            raise base.LocomException(f"Reading input file '{self.settings.input_file}' failed. {e}")

    def _read_template_file(self):
        template_loader = template.Loader()
        self.template = template_loader.load(self.settings.template)

    def _annotate_rows(self):
        row_annotator = annotator.RowAnnotator()
        row_annotator.annotate(self.rules, self.rows)

    def _render(self):
        render_setting = base.RenderSetting()

        render_setting.whitespace_protection = not self.settings.cancel_whitespace_protection
        render_setting.escape_sequence_protection = not self.settings.cancel_escape_sequence_protection

        html_render = render.HtmlRender()
        self.output = html_render.render(render_setting, self._render_elements(), self.template)

    def _render_elements(self):
        return [
            base.RenderElement(base.RenderElementType.title, self.settings.title),
            base.RenderElement(base.RenderElementType.source_file, os.path.abspath(self.settings.input_file)),
            base.RenderElement(base.RenderElementType.description, self.settings.description),
            base.RenderElement(base.RenderElementType.row_number_column, self.settings.row_number_column),
            base.RenderElement(base.RenderElementType.log_column, self.settings.log_column),
            base.RenderElement(base.RenderElementType.comment_column, self._count_comment_column()),
            base.RenderElement(base.RenderElementType.multi_row_column, self.settings.mr_column),
            base.RenderRows(self.rows),
        ]

    def _count_comment_column(self):
        # TODO: May be it should be outside of this class
        return str(100 - (int(self.settings.row_number_column) +
                          int(self.settings.log_column) +
                          2 * int(self.settings.mr_column))
                   )

    def _write_to_output_file(self):
        with open(self._output_file(), "w") as o:
            o.write(self.output)

    def _output_file(self):
        if self.settings.output_file == "":
            right_dot_index = self.settings.input_file.rfind(".")
            file_without_suffix = self.settings.input_file[:right_dot_index]
            file = f"{file_without_suffix}.html"
            return file
        else:
            return self.settings.output_file


import generic_design_patterns as gdp

from . import abstract
from . import replacing_render
from . import row_render

# TODO: Review and rework template. Especially CSS rules.


class HtmlRender:
    def __init__(self):
        element_render_collectors = [gdp.plugin.SubclassPluginCollector(abstract.AbstractElementRender)]
        self.renders_chain = gdp.chain.build(element_render_collectors)

    def render(self, setting, elements, template) -> str:
        output = template

        for element in elements:
            output = self.renders_chain.handle(setting, output, element)

        return output






from setuptools import setup, find_packages

classifiers = [
    'Development Status :: 2 - Pre-Alpha',
    'Intended Audience :: Developers',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
    'Operating System :: OS Independent'
]

with open("README.md", "r") as fh:
    long_description = fh.read()

description = "Log Comments"

setup(
    name='locom',
    version='0.3.4',
    packages=find_packages(),
    package_data={
        "locom": ["template/*.html"],
    },
    url='https://github.com/ShadowCodeCz/locom',
    project_urls={
        'Source': 'https://github.com/ShadowCodeCz/locom',
        'Tracker': 'https://github.com/ShadowCodeCz/locom/issues',
    },
    author='ShadowCodeCz',
    author_email='shadow.code.cz@gmail.com',
    description=description,
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=classifiers,
    keywords='log comments',
    install_requires=['generic-design-patterns', "Pillow"],
    entry_points={
    'console_scripts': [
        'locom=locom:main',
    ],
}
)



from . import abstract
from .. import base


class ReplacingRender(abstract.AbstractElementRender):
    def check(self, setting, template, element) -> bool:
        return True if element.type != base.RenderElementType.rows else False

    def handle(self, setting, template, element):
        return template.replace(self._target_place(element), str(element.text))



import pytest
import argparse
import locom
import os

# TODO: Consider using of tmpdir fixture. Carefull with Travis and --basetemp.


def current_path(filename):
    return os.path.join(os.path.dirname(__file__), filename)


@pytest.fixture()
def cli_arguments():
    output_file = current_path("output.html")

    if os.path.exists(output_file):
        os.remove(output_file)

    arguments = {
        "input_file": current_path("input.txt"),
        "rules_file": current_path("rules.txt"),
        "output_file": output_file,
        "template": "dark",
        "title": "Fake title",
        "description": "Fake description",
        "row_number_column": "3",
        "log_column": "3",
        "mr_column": "2",
        "cancel_whitespace_protection": True,
        "cancel_escape_sequence_protection": True
    }

    mocked_arguments = argparse.Namespace(**arguments)

    yield mocked_arguments

    os.remove(output_file)


def test_cli_create_output_file(cli_arguments):
    locom.cli.run(cli_arguments)
    assert True == os.path.exists(cli_arguments.output_file)


